from ._flight_dates import FlightDates
from ._flight_destinations import FlightDestinations
from ._flight_offers_search import FlightOffersSearch
from ._hotel_offer_search import HotelOfferSearch
from ._hotel_offers_search import HotelOffersSearch
from ._activities import Activities
from ._transfer_offers import TransferOffers

__all__ = ['FlightDates', 'FlightDestinations',
           'HotelOffersSearch', 'HotelOfferSearch',
           'FlightOffersSearch', 'Activities',
           'TransferOffers']



from amadeus.client.decorator import Decorator


class FlightAvailabilities(Decorator, object):
    def post(self, body):
        '''
        Get available seats in different fare classes

        .. code-block:: python

            amadeus.shopping.availability.flight_availabilities.post(body)

        :param body: the parameters to send to the API

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.post(
            '/v1/shopping/availability/flight-availabilities', body)



from amadeus.client.decorator import Decorator
from .analytics import CategoryRatedAreas


class Analytics(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.category_rated_areas = CategoryRatedAreas(client)



from amadeus.namespaces._reference_data import ReferenceData
from amadeus.namespaces._travel import Travel
from amadeus.namespaces._shopping import Shopping
from amadeus.namespaces._e_reputation import EReputation
from amadeus.namespaces._airport import Airport
from amadeus.namespaces._booking import Booking
from amadeus.namespaces._schedule import Schedule
from amadeus.namespaces._analytics import Analytics
from amadeus.namespaces._location import Location
from amadeus.namespaces._airline import Airline
from amadeus.namespaces._ordering import Ordering


class Core(object):
    def __init__(self):
        self.reference_data = ReferenceData(self)
        self.travel = Travel(self)
        self.shopping = Shopping(self)
        self.e_reputation = EReputation(self)
        self.airport = Airport(self)
        self.booking = Booking(self)
        self.schedule = Schedule(self)
        self.analytics = Analytics(self)
        self.location = Location(self)
        self.airline = Airline(self)
        self.ordering = Ordering(self)



from amadeus.client.decorator import Decorator


class Destinations(Decorator, object):
    def get(self, **params):
        '''
        Get airline destinations.

        .. code-block:: python

            amadeus.airline.destinations.get(airlineCode='BA')

        :param airlineCode: the airline code following IATA standard.
            Example: ``"BA"``

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v1/airline/destinations', **params)



from amadeus.client.decorator import Decorator


class Activity(Decorator, object):
    def __init__(self, client, activity_id):
        Decorator.__init__(self, client)
        self.activity_id = activity_id

    def get(self, **params):
        '''
        Returns a single activity from a given id.

        .. code-block:: python

            client.shopping.activities('4615').get()

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v1/shopping/activities/{0}'
                               .format(self.activity_id), **params)



from amadeus.client.decorator import Decorator
from amadeus.reference_data.locations.hotels import ByCity
from amadeus.reference_data.locations.hotels import ByGeocode
from amadeus.reference_data.locations.hotels import ByHotels


class Hotels(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.by_hotels = ByHotels(client)
        self.by_geocode = ByGeocode(client)
        self.by_city = ByCity(client)



from amadeus.client.decorator import Decorator


class CategoryRatedAreas(Decorator, object):
    def get(self, **params):
        '''
        Gets popularity score for location categories

        .. code-block:: python

            amadeus.location.analytics.category_rated_areas.get(
                            latitude=41.397158,
                            longitude=2.160873)

        :param latitude: latitude of geographic location to search around.
            For example: ``41.397158``
        :param longitude: longitude of geographic location to search around.
            For example: ``2.160873``

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v1/location/analytics/category-rated-areas',
                               **params)



from amadeus.client.decorator import Decorator


class HotelOffersSearch(Decorator, object):
    def get(self, **params):
        '''
        Get all offers given hotels

        .. code-block:: python

           amadeus.shopping.hotel_offers_search.get(hotelIds=RTPAR001',
            adults='2')

        :param hotelId: Amadeus Property Code (8 chars), for
            example ``RTPAR001``.

        :param adults: the number of adult passengers with age 12 or older

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v3/shopping/hotel-offers', **params)



from amadeus.client.decorator import Decorator
from amadeus.shopping.flight_offers._prediction import FlightChoicePrediction
from amadeus.shopping.flight_offers._pricing import FlightOffersPrice
from amadeus.shopping.flight_offers._upselling import Upselling


class FlightOffers(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.prediction = FlightChoicePrediction(client)
        self.pricing = FlightOffersPrice(client)
        self.upselling = Upselling(client)



from ._flight_orders import FlightOrders
from ._flight_order import FlightOrder
from ._hotel_bookings import HotelBookings
from ._hotel_orders import HotelOrders

__all__ = ['FlightOrders', 'FlightOrder', 'HotelBookings', 'HotelOrders']



from amadeus.client.decorator import Decorator

from urllib.parse import urlencode


class Cancellation(Decorator, object):
    def __init__(self, client, order_id):
        Decorator.__init__(self, client)
        self.order_id = order_id

    def post(self, body, **params):
        '''
        Cancels a transfer reservation

        .. code-block:: python

            amadeus.ordering.transfer_order(order_id).transfers.cancellation.post(body,
            confirmNbr=confirm_nbr)

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        url = '/v1/ordering/transfer-orders/{0}/transfers/cancellation?'.format(
            self.order_id
        )
        return self.client.post(url + urlencode(params), body)



from amadeus.client.decorator import Decorator
from amadeus.ordering.transfer_orders import Transfers


class TransferOrder(Decorator, object):
    def __init__(self, client, order_id):
        Decorator.__init__(self, client)
        self.transfers = Transfers(client, order_id)



from amadeus.client.decorator import Decorator

from urllib.parse import urlencode


class TransferOrders(Decorator, object):
    def post(self, body, **params):
        '''
        Performs the final booking for a chosen transfer

        .. code-block:: python

            amadeus.ordering.transfer_orders.post(body, offerId=offer_id)

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        url = '/v1/ordering/transfer-orders?'
        return self.client.post(url + urlencode(params), body)



from amadeus.client.decorator import Decorator


class HotelOrders(Decorator, object):
    def post(self,
             guests,
             travel_agent,
             room_associations=[],
             payment={},
             arrival_information={}):
        '''
        Book hotel(s) via Hotel Booking API V2

        .. code-block:: python

            amadeus.booking.hotel_orders.post(guests,
                                              travel_agent,
                                              room_associations,
                                              payment,
                                              arrival_information)

        The parameters guests and room_associations can be passed as dictionary
        or list of dictionaries. If they are dictionary in this method they are
        converted to a list of dictionaries.

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        guests_info = []
        room_associations_info = []
        if not isinstance(guests, list):
            guests_info.append(guests)
        else:
            guests_info.extend(guests)
        if not isinstance(room_associations, list):
            room_associations_info.append(room_associations)
        else:
            room_associations_info.extend(room_associations)
        body = {'data': {'type': 'hotel-order',
                         'guests': guests_info,
                         'travelAgent': travel_agent,
                         'roomAssociations': room_associations_info,
                         'arrivalInformation': arrival_information,
                         'payment': payment}}
        return self.client.post('/v2/booking/hotel-orders', body)



from amadeus.client.decorator import Decorator
from .availability import FlightAvailabilities


class Availability(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.flight_availabilities = FlightAvailabilities(client)



from amadeus.client.decorator import Decorator


class BySquare(Decorator, object):
    def get(self, **params):
        '''
        Returns a list of relevant point of interests
            around a defined square (4 points).

        .. code-block:: python


            client.reference_data.locations.points_of_interest.by_square.get(
                north=41.397158,
                west=2.160873,
                south=41.394582,
                east=2.177181
            )

        :param north: latitude north of bounding box.
            For example: ``41.397158``
        :param west: longitude west of bounding box.
            For example: ``2.160873``
        :param south: latitude south of bounding box.
            For example: ``41.394582``
        :param east: longitude east of bounding box.
            For example: ``2.177181``

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get(
            '/v1/reference-data/locations/pois/by-square', **params)



from amadeus.client.decorator import Decorator


class ItineraryPriceMetrics(Decorator, object):

    def get(self, **params):
        '''
        Returns itinerary price metrics by search criteria

        .. code-block:: python

            amadeus.analytics.itinerary_price_metrics.get(
                                            originIataCode='MAD',
                                            destinationIataCode='CDG',
                                            departureDate='2021-03-21')

        :param originIataCode: IATA code of the origin city, for
            example ``"BOS"`` for Boston.

        :param destinationIataCode: IATA code of the destination city,
            for example ``"ATH"`` for Athens.

        :param departureDate: scheduled departure date of the flight,
            local to the departure airport, format YYYY-MM-DD

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v1/analytics/itinerary-price-metrics', **params)



from amadeus.client.decorator import Decorator


class DirectDestinations(Decorator, object):
    def get(self, **params):
        '''
        Returns airport direct routes.

        .. code-block:: python

            amadeus.airport.direct_destinations.get(
                            departureAirportCode='BLR')

        :param departureAirportCode: the departure Airport code following
            IATA standard. ``"BLR"``, for example for Bengaluru

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v1/airport/direct-destinations', **params)



from amadeus.client.decorator import Decorator


class FlightOrders(Decorator, object):
    def post(self, flight, travelers):
        '''
        Books a flight

        .. code-block:: python

            amadeus.booking.flight_orders.post(flight, travelers)

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        flight_offers = []
        if not isinstance(flight, list):
            flight_offers.append(flight)
        else:
            flight_offers.extend(flight)
        travelers_info = []
        if not isinstance(travelers, list):
            travelers_info.append(travelers)
        else:
            travelers_info.extend(travelers)
        body = {'data': {'type': 'flight-order', 'flightOffers': flight_offers,
                         'travelers': travelers_info}}
        return self.client.post('/v1/booking/flight-orders', body)



from amadeus.client.decorator import Decorator
from amadeus.airline._destinations import Destinations


class Airline(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.destinations = Destinations(client)



from amadeus.client.decorator import Decorator


class HotelSentiments(Decorator, object):
    def get(self, **params):
        '''
        Provides ratings and sentiments scores for hotels

        .. code-block:: python

            amadeus.e_reputation.hotel_sentiments.get(hotelIds='TELONMFS,ADNYCCTB'])

        :param hotelIds: comma separated string list of amadeus hotel Ids (max
        3). These Ids are found in the Hotel Search response. ``"RDLON308"``,
        for example for the Radisson Blu Hampshire hotel.

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v2/e-reputation/hotel-sentiments', **params)



from amadeus.client.decorator import Decorator
from amadeus.ordering.transfer_orders.transfers import Cancellation


class Transfers(Decorator, object):
    def __init__(self, client, order_id):
        Decorator.__init__(self, client)
        self.cancellation = Cancellation(client, order_id)



from amadeus.client.decorator import Decorator

from urllib.parse import urlencode


class FlightOffersPrice(Decorator, object):
    def post(self, body, **params):
        '''
        Gets a confirmed price and availability of a flight

        .. code-block:: python

            amadeus.shopping.flight_offers.pricing.post(body, params)

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        url = '/v1/shopping/flight-offers/pricing?'
        flight_offers = []
        if not isinstance(body, list):
            flight_offers.append(body)
        else:
            flight_offers.extend(body)
        return self.client.post(url + urlencode(params),
                                {'data':
                                {'type': 'flight-offers-pricing',
                                    'flightOffers': flight_offers}})



from amadeus.client.decorator import Decorator


class RecommendedLocations(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)

    def get(self, **params):
        '''
        Returns a list of destination recommendations

        .. code-block:: python


            client.reference_data.recommended_locations.get(
                cityCodes='PAR'
                travelerCountryCode='FR'
            )

        :param cityCodes: city used by the algorithm to recommend new destinations
            For example: ``PAR``
        :param travelerCountryCode: origin country following IATA standard
            For example: ``FR``

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get(
            '/v1/reference-data/recommended-locations', **params)



from amadeus.client.decorator import Decorator
from amadeus.analytics._itinerary_price_metrics import ItineraryPriceMetrics


class Analytics(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.itinerary_price_metrics = ItineraryPriceMetrics(client)



#
class Hotel(object):
    '''
    A list of hotel sub types, as used in Hotel Name Autocomplete

    .. code-block:: python


        from amadeus import Hotel

        amadeus.reference_data.locations.hotel.get(
            keyword='PARI',
            subType=[Hotel.HOTEL_LEISURE, Hotel.HOTEL_GDS]
        )

    :cvar HOTEL_LEISURE: ``"HOTEL_LEISURE"``
    :cvar HOTEL_GDS: ``"HOTEL_GDS"``
    '''
    # Hotel Leisure
    HOTEL_LEISURE = 'HOTEL_LEISURE'
    # Hotel GDS
    HOTEL_GDS = 'HOTEL_GDS'



from amadeus.client.decorator import Decorator
from amadeus.travel._analytics import Analytics
from amadeus.travel._predictions import Predictions
from amadeus.travel._trip_parser import TripParser
from amadeus.travel._encoder import from_file, from_base64


class Travel(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.analytics = Analytics(client)
        self.predictions = Predictions(client)
        self.trip_parser = TripParser(client)

    @staticmethod
    def from_file(file):
        return from_file(file)

    @staticmethod
    def from_base64(base64):
        return from_base64(base64)



from amadeus.client.decorator import Decorator


class AirportOnTime(Decorator, object):
    def get(self, **params):
        '''
        Returns a percentage of on-time flight departures

        .. code-block:: python

            amadeus.airport.predictions.on_time.get(
                            airportCode='JFK',
                            date='2020-09-01')

        :param airportCode: the City/Airport IATA code from which
            the flight will depart. ``"NYC"``, for example for New York

        :param date: the date on which to fly out, in `YYYY-MM-DD` format

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v1/airport/predictions/on-time', **params)



from amadeus.client.decorator import Decorator


class ByCity(Decorator, object):
    def get(self, **params):
        '''
        Searches for hotel in a given city.

        .. code-block:: python


            amadeus.reference_data.locations.hotels.by_city.get(
                cityCode='PAR')

        :param cityCode: the City IATA code for which to find a hotel, for
            example ``"PAR"`` for Paris.

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get(
            '/v1/reference-data/locations/hotels/by-city', **params)



from amadeus.client.decorator import Decorator


class FlightChoicePrediction(Decorator, object):
    def post(self, body):
        '''
        Forecast traveler choices in the context of search & shopping.

        .. code-block:: python

            amadeus.shopping.flight_offers.prediction.post(
                amadeus.shopping.flight_offers_search.get(
                originLocationCode='SYD',
                destinationLocationCode='BKK',
                departureDate='2020-11-01',
                adults=1).result
        )

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.post('/v2/shopping/flight-offers/prediction', body)



from amadeus.booking._flight_orders import FlightOrders
from amadeus.booking._flight_order import FlightOrder
from amadeus.booking._hotel_bookings import HotelBookings
from amadeus.booking._hotel_orders import HotelOrders
from amadeus.client.decorator import Decorator


class Booking(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.flight_orders = FlightOrders(client)
        self.hotel_bookings = HotelBookings(client)
        self.hotel_orders = HotelOrders(client)

    def flight_order(self, flight_order_id):
        return FlightOrder(self.client, flight_order_id)


__all__ = ['FlightOrders', 'FlightOrder', 'HotelBookings', 'HotelOrders']



from amadeus.client.decorator import Decorator
from amadeus.ordering._transfer_orders import TransferOrders
from amadeus.ordering._transfer_order import TransferOrder


class Ordering(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.transfer_orders = TransferOrders(client)

    def transfer_order(self, order_id):
        return TransferOrder(self.client, order_id)


__all__ = ['TransferOrders', 'TransferOrder']



from amadeus.client.decorator import Decorator
from amadeus.shopping._flight_dates import FlightDates
from amadeus.shopping._flight_destinations import FlightDestinations
from amadeus.shopping._flight_offers import FlightOffers
from amadeus.shopping._flight_offers_search import FlightOffersSearch
from amadeus.shopping._seatmaps import Seatmaps
from amadeus.shopping._activities import Activities
from amadeus.shopping._activity import Activity
from amadeus.shopping._availability import Availability
from amadeus.shopping._hotel_offer_search import HotelOfferSearch
from amadeus.shopping._hotel_offers_search import HotelOffersSearch
from amadeus.shopping._transfer_offers import TransferOffers


class Shopping(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.flight_dates = FlightDates(client)
        self.flight_destinations = FlightDestinations(client)
        self.flight_offers = FlightOffers(client)
        self.flight_offers_search = FlightOffersSearch(client)
        self.seatmaps = Seatmaps(client)
        self.activities = Activities(client)
        self.availability = Availability(client)
        self.hotel_offers_search = HotelOffersSearch(client)
        self.transfer_offers = TransferOffers(client)

    def hotel_offer_search(self, offer_id):
        return HotelOfferSearch(self.client, offer_id)

    def activity(self, activity_id):
        return Activity(self.client, activity_id)


__all__ = ['FlightDates', 'FlightDestinations', 'FlightOffers',
           'FlightOffersSearch', 'Availability']



from amadeus.client.decorator import Decorator


class Hotel(Decorator, object):
    def get(self, **params):
        '''
        Returns a list of hotels matching a given keyword.

        .. code-block:: python


            amadeus.reference_data.locations.hotel.get(
                keyword='PARI',
                subType=[Hotel.HOTEL_LEISURE, Hotel.HOTEL_GDS]
            )

        :param keyword: location query keyword.
            For example: ``PARI``
        :param subType: category of search.
            For example: ``[Hotel.HOTEL_LEISURE, Hotel.HOTEL_GDS]``

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get(
            '/v1/reference-data/locations/hotel', **params)



from amadeus.client.decorator import Decorator
from amadeus.e_reputation._hotel_sentiments import HotelSentiments


class EReputation(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.hotel_sentiments = HotelSentiments(client)



from amadeus.client.decorator import Decorator
from amadeus.shopping.activities._by_square \
    import BySquare


class Activities(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.by_square = BySquare(client)

    def get(self, **params):
        '''
        Returns activities for a given location

        .. code-block:: python


            client.shopping.activities.get(
                longitude=2.160873,
                latitude=41.397158
            )

        :param latitude: latitude of geographic location to search around.
            For example: ``41.397158``
        :param longitude: longitude of geographic location to search around.
            For example: ``2.160873``

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get(
            '/v1/shopping/activities', **params)



from amadeus.client.decorator import Decorator


class Cities(Decorator, object):
    def get(self, **params):
        '''
        Returns cities that match a specific word or letters.

        .. code-block:: python


            amadeus.reference_data.locations.cities.get(
                keyword='PARI'
            )

        :param keyword: location query keyword.
            For example: ``PARI``

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get(
            '/v1/reference-data/locations/cities', **params)



from amadeus.client.decorator import Decorator


class ByHotels(Decorator, object):
    def get(self, **params):
        '''
        Searches for hotel using it's unique id.

        .. code-block:: python


            amadeus.reference_data.locations.hotels.by_hotels.get(
                hotelIds=["ADPAR001"])

        :param hotelIds: Amadeus Property Codes (8 chars)
            For example: ``["ADPAR001"]``

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        for key, value in params.items():
            if isinstance(value, list):
                params[key] = ','.join(value)
        return self.client.get(
            '/v1/reference-data/locations/hotels/by-hotels', **params)



from amadeus.client.decorator import Decorator
from .predictions import TripPurpose, FlightDelay


class Predictions(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.trip_purpose = TripPurpose(client)
        self.flight_delay = FlightDelay(client)



from amadeus.client.decorator import Decorator


class FlightDelay(Decorator, object):
    def get(self, **params):
        '''
        Forecast the chances for a flight to be delayed

        .. code-block:: python

        amadeus.travel.predictions.flight_delay.get(originLocationCode='NCE',
                                                       destinationLocationCode='IST',
                                                       departureDate='2020-08-01',
                                                       departureTime='18:20:00',
                                                       arrivalDate='2020-08-01',
                                                       arrivalTime='22:15:00',
                                                       aircraftCode='321',
                                                       carrierCode='TK',
                                                       flightNumber='1816',
                                                       duration='PT31H10M')

        :param originLocationCode: the City/Airport IATA code from which
            the flight will depart. ``"NYC"``, for example for New York

        :param destinationLocationCode: the City/Airport IATA code to which
            the flight is going. ``"MAD"``, for example for Madrid

        :param departureDate: the date on which the traveler departs
            from the origin, in `YYYY-MM-DD` format

        :param departureTime: local time on which to fly out,
            in `HH:MM:SS` format

        :param arrivalDate: the date on which the traveler arrives
            to the destination, in `YYYY-MM-DD` format

        :param arrivalTime: local time on which the traveler arrives
            to the destination, in `HH:MM:SS` format

        :param aircraftCode: IATA aircraft code

        :param carrierCode: airline / carrier code

        :param flightNumber: flight number as assigned by the carrier

        :param duration: flight duration,
            in `PnYnMnDTnHnMnS` format e.g. PT2H10M

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v1/travel/predictions/flight-delay', **params)
